-- Addon
local modName = ...;
local ip = CreateFrame("Frame",modName,UIParent,BackdropTemplateMixin and "BackdropTemplate");	-- 9.0.1: Using BackdropTemplate

-- Global Chat Message Function
function AzMsg(msg) DEFAULT_CHAT_FRAME:AddMessage(tostring(msg):gsub("|1","|cffffff80"):gsub("|2","|cffffffff"),0.5,0.75,1.0); end

-- Config
local DefaultConfig = {
	topAnchor = false,		-- is panel on top or not
	defaultTip = false,		-- default anchored tips
	iconSize = 18,			-- icon size
	resizeDelta = 6,		-- size difference before resize
};
local cfg = DefaultConfig;

ip.plugins = {};

--------------------------------------------------------------------------------------------------------
--                                           Slash Handling                                           --
--------------------------------------------------------------------------------------------------------

-- Slash Commands -- Array indices are used for command help. Commands starting with underscore are meant for debugging
ip.slashCommands = {
	-- Show / Hide
	" |2show|r = Toggles the panel being shown or not",
	show = function(cmd)
		ip:SetShown(not ip:IsShown());
	end,
	-- Swap Anchor
	" |2anchor|r = Swap anchor position between top and bottom",
	anchor = function(cmd)
		cfg.topAnchor = not cfg.topAnchor;
		ip:AdjustAnchors();
	end,
	-- Tip Location
	" |2tip|r = Toggles between plugin anchored tip or default anchored tip",
	tip = function(cmd)
		cfg.defaultTip = not cfg.defaultTip;
	end,
	-- Icon Size
	" |2size|r = Adjusts icon size as well as panel height (default = 18)",
	size = function(cmd)
		cfg.iconSize = max(1,tonumber(cmd) or cfg.iconSize);
		ip:AdjustWidths();
		ip:AdjustAnchors();
	end,
};

local function HandleSlashCommand(cmd,commandTable)
	-- Extract Paramters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Check Param Function
	if (commandTable[param1]) then
		commandTable[param1](param2)
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg(format("----- |2%s|r |1%s|r ----- |1%.2f |2kb|r -----",modName,GetAddOnMetadata(modName,"Version"),GetAddOnMemoryUsage(modName)));
		AzMsg("The following |2parameters|r are valid for this addon:");
		for index, help in ipairs(commandTable) do
			AzMsg(help);
		end
	end
end

_G["SLASH_"..modName.."1"] = "/ip";
_G["SLASH_"..modName.."2"] = "/infopanel";
SlashCmdList[modName] = function(cmd) HandleSlashCommand(cmd,ip.slashCommands) end

--------------------------------------------------------------------------------------------------------
--                                       Plugin Helper Functions                                      --
--------------------------------------------------------------------------------------------------------

-- Standard OnEvent Script
local function Default_OnEvent_Forwarder(self,event,...)
	if (self[event]) then
		self[event](self,event,...);
	end
end

-- Standard OnEnter Script
local function Default_OnEnter(self)
	if (self.OnShowTooltip) then
		if (cfg.defaultTip) then
			GameTooltip_SetDefaultAnchor(GameTooltip,self);
		else
			local a1 = cfg.topAnchor and "TOP" or "BOTTOM";
			local a2 = cfg.topAnchor and "BOTTOM" or"TOP";
			local b = self.isRightAligned and "RIGHT" or "LEFT";

			GameTooltip:SetOwner(self,"ANCHOR_NONE");
			GameTooltip:SetPoint(a1..b,self,a2..b);
		end
		self:OnShowTooltip(GameTooltip);
		GameTooltip:Show();
	end
end

-- Standard OnLeave Script
local function HideGTT()
	GameTooltip:Hide();
end

-- Quick SetText for Plugins to use
local function SetText(self,text,...)
	-- text
	if (select("#",...) > 0) then
		self.text:SetFormattedText(text,...);
	else
		self.text:SetText(text);
	end
	-- width
	local oldWidth = self:GetWidth();
	local newWidth = (self.text:GetWidth() + cfg.iconSize + 8);
	if (abs(oldWidth - newWidth) > cfg.resizeDelta) then
		self:SetWidth(newWidth);
	end
	-- tip
	if (GameTooltip:IsOwned(self)) and (self:GetScript("OnEnter")) then
		self:GetScript("OnEnter")(self);
	end
end

-- SetTextColor
local function SetTextColor(self,value,lowBoundry,highBoundry)
	if (value > highBoundry) then
		self.text:SetTextColor(0,1,0);
	elseif (value > lowBoundry) then
		self.text:SetTextColor(1,1,0);
	else
		self.text:SetTextColor(1,0,0);
	end
end

-- Add Plugin
function ip:AddPlugin(align)
	local frame = CreateFrame("Button",nil,ip);

	--frame:SetSize(cfg.iconSize,ip:GetHeight() - 2);
	frame:EnableMouse(1);
	frame:RegisterForClicks("AnyUp");
	frame:SetScript("OnEvent",Default_OnEvent_Forwarder);
	frame:SetScript("OnEnter",Default_OnEnter);
	frame:SetScript("OnLeave",HideGTT);

--frame:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tileSize = 8, edgeSize = 8, insets = { left = 0, right = 0, top = 0, bottom = 0 } });
--frame:SetBackdropColor(0,1,0);

	-- Icon + Text
	frame.icon = frame:CreateTexture(nil,"OVERLAY");
	frame.icon:SetPoint("LEFT");
	frame.icon:SetWidth(cfg.iconSize);
	frame.icon:SetHeight(cfg.iconSize);
	frame.icon:SetTexCoord(0.07,0.93,0.07,0.93);

	frame.text = frame:CreateFontString(nil,"OVERLAY","GameFontHighlight");
	frame.text:SetPoint("LEFT",frame.icon,"RIGHT",4,0);

	-- Functions
	frame.SetText = SetText;
	frame.SetTextColor = SetTextColor;

	-- Anchor
	frame:SetPoint("TOP");
	frame:SetPoint("BOTTOM");
	if (align == "right") then
		frame.isRightAligned = 1;
		frame:SetPoint("RIGHT",-4,0);
	elseif (#ip.plugins > 0) then
		frame:SetPoint("LEFT",ip.plugins[#ip.plugins],"RIGHT",6,0);
	else
		frame:SetPoint("LEFT",4,0);
	end

	-- Return
	ip.plugins[#ip.plugins + 1] = frame;
	return frame;
end

-- Sends an event to all plugins
function ip:SendPluginEvent(event,...)
	for _, plugin in ipairs(self.plugins) do
		if (not plugin.disabled) and (plugin[event]) then
			plugin[event](plugin,...);
		end
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Core Functions                                           --
--------------------------------------------------------------------------------------------------------

-- AdjustAnchors
function ip:AdjustAnchors()
	-- InfoPanel
	local offset = 2;
	ip:ClearAllPoints();
	if (cfg.topAnchor) then
		ip:SetPoint("TOPLEFT",-offset,offset);
		ip:SetPoint("TOPRIGHT",offset,offset);
	else
		ip:SetPoint("BOTTOMLEFT",-offset,-offset);
		ip:SetPoint("BOTTOMRIGHT",offset,-offset);
	end
--[[
	-- MainMenuBar -- we cannot move this without causing taint
	if (MainMenuBar:IsShown()) then
		--MainMenuBar:ClearAllPoints();
		MainMenuBar.GetYOffset = function(self) return self.yOffset + (cfg.topAnchor and 0 or ip:GetHeight() - 2) end;

		if (cfg.topAnchor) then
			MainMenuBar:SetPoint("BOTTOM");
		else
			MainMenuBar:SetPoint("BOTTOM",ip,"TOP",0,13);
		end
	end
--]]
end

-- Adjust Widths
function ip:AdjustWidths()
	ip:SetHeight(cfg.iconSize + 8);
	for index, plugin in ipairs(ip.plugins) do
		plugin.icon:SetWidth(cfg.iconSize);
		plugin.icon:SetHeight(cfg.iconSize);
		plugin:SetWidth(plugin.text:GetWidth() + cfg.iconSize + 8);
	end
end

-- Variables Loaded [One-Time-Event]
function ip:VARIABLES_LOADED(event,...)
	if (not InfoPanel_Config) then
		InfoPanel_Config = {};
	end
	cfg = setmetatable(InfoPanel_Config,{ __index = DefaultConfig });
	self:SendPluginEvent("OnLoad",cfg);
	self:AdjustWidths();
	self:AdjustAnchors();
	self[event] = nil;
end

-- Setup the Main Frame
ip:SetFrameStrata("MEDIUM");
ip:SetToplevel(true);
ip:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 8, insets = { left = 2, right = 2, top = 2, bottom = 2 } });
ip:SetBackdropColor(0.1,0.22,0.35,1);
ip:SetBackdropBorderColor(0.1,0.1,0.1,1);
ip:Show();
ip:SetScript("OnEvent",Default_OnEvent_Forwarder);
ip:RegisterEvent("VARIABLES_LOADED");