local plugin = InfoPanel:AddPlugin();
local cfg;

local UPDATE_INTERVAL = 0.5;
local nextUpdate = 0;
local zoneChange, lastZoneChange;

--------------------------------------------------------------------------------------------------------
--                                            Helper Class                                            --
--------------------------------------------------------------------------------------------------------

--[[
	GetPlayerMapPosition Without Garbage!
	-------------------------------------
	Replacement for the now removed function GetPlayerMapPosition()
	It would be possible to use C_Map.GetPlayerMapPosition(unit) for this task.
	However, since that function returns a new table on each call, it causes lots of garbage.
	This is a workaround which caches the coordinates for the current map,
	and normalizes the player coordinates within them.
	More Info: http://www.wowinterface.com/forums/showthread.php?t=56290
	Usage: PlayerMapPosition:GetXY("player")
--]]
local PlayerMapPosition = {
	player = CreateVector2D(0,0),
	topLeft = CreateVector2D(0,0),
	bottomRight = CreateVector2D(1,1),
	GetMapVectors = function(self,mapID)
		local mv = self[mapID];
		if (not mv) then
			self[mapID] = {};
			mv = self[mapID];
			local _;
			_, mv[1] = C_Map.GetWorldPosFromMapPos(mapID,self.topLeft);
			_, mv[2] = C_Map.GetWorldPosFromMapPos(mapID,self.bottomRight);
			if (not mv[1]) or (not mv[2]) then
				return;
			end
			mv[2]:Subtract(mv[1]);
		end
		return mv[1], mv[2];
	end,
	GetXY = function(self,unit)
		local mapID = C_Map.GetBestMapForUnit(unit);
		if (not mapID) then
			return 0, 0;
		end

		local a, b = self:GetMapVectors(mapID);
		if (not a) or (not b) then
			return 0, 0;
		end
		local player = self.player;

		player:SetXY(UnitPosition(unit));
		player:Subtract(a);

		local x, y = player:GetXY();
		return (y / b.y), (x / b.x);
	end
};

--------------------------------------------------------------------------------------------------------
--                                             Functions                                              --
--------------------------------------------------------------------------------------------------------

-- OnEvent
local function OnEvent(self,event)
	if (not zoneChange) or (GetTime() - zoneChange > 2) then
		lastZoneChange = zoneChange;
		zoneChange = GetTime();
	end
	if (cfg.zoneText) then
		self:SetText(GetMinimapZoneText());
	end
end

-- OnUpdate
local function OnUpdate(self,elapsed)
	nextUpdate = (nextUpdate - elapsed);
	if (nextUpdate <= 0) then
		local x, y = PlayerMapPosition:GetXY("player");
		if (x == 0 and y == 0) then
			self:SetText("n/a");
		else
			self:SetText("%.1f |cffffff00|||r %.1f",x * 100,y * 100);
		end
		nextUpdate = UPDATE_INTERVAL;
	end
end

-- OnShowTooltip
function plugin:OnShowTooltip(tooltip)
	tooltip:AddLine("Zone Info",1,1,1);

	tooltip:AddDoubleLine("Zone",GetZoneText(),nil,nil,nil,1,1,1);
	tooltip:AddDoubleLine("Real Zone",GetRealZoneText(),nil,nil,nil,1,1,1);
	--tooltip:AddDoubleLine("Sub Zone",GetSubZoneText(),nil,nil,nil,1,1,1); -- Az: Sometimes returns nothing
	tooltip:AddDoubleLine("Minimap Zone",GetMinimapZoneText(),nil,nil,nil,1,1,1);	-- seems to return SubZoneText if valid, otherwise (Real)ZoneText

	local pvpType, isFFA, faction = GetZonePVPInfo();
	if (pvpType) then
		tooltip:AddLine(" ");
		tooltip:AddDoubleLine("Zone PvP Type",(pvpType or ""):gsub(".",strupper,1),nil,nil,nil,1,1,1);
		tooltip:AddDoubleLine("Zone Free for All",tostring(isFFA),nil,nil,nil,1,1,1);
		tooltip:AddDoubleLine("Zone Faction",tostring(faction),nil,nil,nil,1,1,1);
	end

	local inInstance, instanceType = IsInInstance();
	tooltip:AddLine(" ");
	tooltip:AddDoubleLine("In Instance",tostring(inInstance and "true" or "false"),nil,nil,nil,1,1,1);
	tooltip:AddDoubleLine("Instance Type",tostring(instanceType),nil,nil,nil,1,1,1);

	tooltip:AddLine(" ");
	tooltip:AddDoubleLine("This Zone's Timer",InfoPanel.FormatTime2(GetTime() - zoneChange),nil,nil,nil,1,1,1);
	if (lastZoneChange) then
		tooltip:AddDoubleLine("Last Zone's Timer",InfoPanel.FormatTime2(zoneChange - lastZoneChange),nil,nil,nil,1,1,1);
	end

	tooltip:AddLine(" ");
	tooltip:AddDoubleLine("Raw Map Position",format("%.1f, %.1f",UnitPosition("player")),nil,nil,nil,1,1,1);
end

-- plugin event: OnLoad
function plugin:OnLoad(config)
	cfg = config;
	self:OnApplyConfig(cfg);
end

-- apply configs
function plugin:OnApplyConfig(config)
	if (cfg.zoneText) then
		plugin:SetScript("OnUpdate",nil);
		OnEvent(plugin,"ZONE_CHANGED_NEW_AREA");
	else
		plugin:SetScript("OnUpdate",OnUpdate);
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Slash Commands                                           --
--------------------------------------------------------------------------------------------------------

InfoPanel.slashCommands[#InfoPanel.slashCommands + 1] = " |2zonetext|r = Toggles zonetext being coords or location";
InfoPanel.slashCommands.zonetext = function(cmd)
	cfg.zoneText = not cfg.zoneText;
	plugin:OnApplyConfig(cfg);
end

--------------------------------------------------------------------------------------------------------
--                                              Finalize                                              --
--------------------------------------------------------------------------------------------------------

plugin.icon:SetTexture("Interface/Icons/Achievement_Zone_Northrend_01");
--plugin.icon:SetTexture("Interface/Icons/Achievement_Zone_EasternKingdoms_01");

plugin:SetScript("OnClick",function() ToggleFrame(WorldMapFrame); end);
plugin:SetScript("OnEvent",OnEvent);
plugin:RegisterEvent("PLAYER_ENTERING_WORLD");
plugin:RegisterEvent("ZONE_CHANGED_NEW_AREA");